#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2015 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Contact view components
"""

from zope.container.interfaces import INameChooser

from z3c.form import field
from z3c.form import form

from schooltool.app.interfaces import ISchoolToolApplication
from schooltool.common import SchoolToolMessage as STMessage
from schooltool.contact.browser.contact import FlourishPersonContactAddView
from schooltool.contact.browser.contact import FlourishContactEditView
from schooltool.contact.contact import Contact
from schooltool.contact.interfaces import IContactDemographicsFields
from schooltool.contact.interfaces import IContactContainer
from schooltool.contact.interfaces import IContact


class RisingContactFormBase(object):

    @property
    def fields(self):
        result = field.Fields(IContact)
        result += self.getDemoFields()
        return result

    def getDemoFields(self):
        fields = field.Fields()
        dfs = IContactDemographicsFields(ISchoolToolApplication(None))
        keys = []
        for field_desc in dfs.filter_keys(keys):
            fields += field_desc.makeField()
        return fields

    def buildFieldsetGroups(self):
        self.fieldset_groups = {
            'full_name': (
                STMessage('Full Name'),
                ['first_name', 'last_name']),
            'demographics': (
                STMessage('Demographics'), list(self.getDemoFields())),
        }
        self.fieldset_order = (
            'full_name', 'demographics')


class RisingContactEditView(RisingContactFormBase,
                            FlourishContactEditView):

    pass


class RisingPersonContactAddView(RisingContactFormBase,
                                 FlourishPersonContactAddView):

    def get_name(self, contact):
        contact_container = IContactContainer(ISchoolToolApplication(None))
        return INameChooser(contact_container).chooseName('', contact)

    def create(self, data):
        contact = Contact()
        self.contact_name = self.get_name(contact)
        contact.__name__ = self.contact_name
        form.applyChanges(self, contact, data)
        return contact

    def add(self, contact):
        """Add `contact` to the container and assign it to the person."""
        contact_container = IContactContainer(ISchoolToolApplication(None))
        contact_container[self.contact_name] = contact
        self.added = contact
        return contact
