#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2015 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Journey view components
"""

import urllib2
from datetime import datetime
from datetime import timedelta
import socket

from zope.cachedescriptors.property import Lazy
from zope.component import getUtility
from zope.traversing.browser.absoluteurl import absoluteURL

from z3c.form import field
from z3c.form.browser.checkbox import SingleCheckBoxFieldWidget
import zc.resourcelibrary

from schooltool.stream.browser.stream import StreamAddView
from schooltool.stream.browser.stream import StreamEditView
from schooltool.skin import flourish
from schooltool.task.tasks import get_message_by_id
from schooltool.task.tasks import query_message

from schooltool.rising.interfaces import IJourneyAccessUtility
from schooltool.rising.interfaces import IAttendanceGroup
from schooltool.rising.interfaces import IJourneyPreferences
from schooltool.rising.journey import HeadRequest
from schooltool.rising.journey import SyncAttendanceTask


class JourneyIntegrationOverview(flourish.page.Content):

    body_template = flourish.templates.File(
        'templates/journey_integration_overview.pt')

    @Lazy
    def last_sync(self):
        return IJourneyPreferences(self.context).last_sync

    @Lazy
    def next_sync(self):
        if self.last_sync is not None:
            return self.last_sync + timedelta(days=1)
        return datetime.now() + timedelta(days=1)


class JourneyAdminView(flourish.page.Page):

    pass


class JourneyStatusOverview(flourish.page.Viewlet):

    template = flourish.templates.Inline('''
    <h3 i18n:domain="schooltool.rising" i18n:translate="">
      Journey Status
    </h3>
    <div>
      <tal:block content="structure view/view/providers/ajax/journey_status" />
    </div>
    ''')


class JourneyStatus(flourish.ajax.AJAXPart):

    container_id = 'journey-status'
    error = ''
    duration = None
    timeout = 30

    template = flourish.templates.File('templates/journey_status.pt')

    def update(self):
        zc.resourcelibrary.need('schooltool.table')
        if self.fromPublication:
            self.error, self.duration = self.connect()

    @Lazy
    def server(self):
        return getUtility(IJourneyAccessUtility).account_url

    def connect(self):
        utility = getUtility(IJourneyAccessUtility)
        request = HeadRequest('%s/datamodel.json' % self.server)
        request.add_header('Authorization', utility.authorization_header)
        start = datetime.utcnow()
        try:
            response = urllib2.urlopen(request, timeout=self.timeout)
        except (urllib2.URLError,) as exc:
            end = datetime.utcnow()
            return exc.reason, (end - start).total_seconds()
        except (urllib2.HTTPError,) as exc:
            fp = exc
        except (socket.timeout,) as exc:
            return exc.message, None
        else:
            fp = response
        errors = []
        if fp.code >= 400:
            errors.append(fp.msg)
        end = datetime.utcnow()
        return '. '.join(errors), (end - start).total_seconds()

    def format_seconds(self, value):
        return '%.2f' % value


class RisingStreamForm(object):

    @property
    def fields(self):
        default = super(RisingStreamForm, self).fields
        default += field.Fields(IAttendanceGroup)
        default['attendance_group'].widgetFactory = SingleCheckBoxFieldWidget
        return default.select('title', 'description', 'attendance_group')


class RisingStreamAddView(RisingStreamForm, StreamAddView):

    def create(self, data):
        self._attendance_group = data['attendance_group']
        return super(RisingStreamAddView, self).create(data)

    def add(self, stream):
        stream = super(RisingStreamAddView, self).add(stream)
        IAttendanceGroup(stream).attendance_group = self._attendance_group
        return stream


class RisingStreamEditView(RisingStreamForm, StreamEditView):

    pass


class JourneyActionsLinks(flourish.page.RefineLinksViewlet):

    pass


class SyncAttendanceView(flourish.page.Page):

    message_b64 = None

    @property
    def message(self):
        if not self.message_b64:
            return None
        message_id = self.message_b64.decode('base64').decode('utf-8')
        message = get_message_by_id(message_id)
        return message

    def __call__(self):
        self.scheduleSync()
        self.request.response.redirect(self.nextURL())

    def nextURL(self):
        message = self.message
        if message is not None:
            return absoluteURL(message, self.request)
        url = absoluteURL(self.context, self.request)
        return '%s/journey' % url

    def scheduleSync(self):
        task = SyncAttendanceTask(self.context)
        task.schedule(self.request)
#        message = query_message(task)
#        message_id = message.__name__.encode('utf-8').encode('base64').strip()
#        self.message_b64 = message_id
