#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2015 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Person view components
"""

from zope.component import getUtility
from zope.interface import Interface
from zope.schema import Choice
from zope.traversing.browser.absoluteurl import absoluteURL

from z3c.form import field

from schooltool.ark.browser.person import StudentAddView as ArkStudentAddView
from schooltool.term.interfaces import IDateManager
from schooltool.skin import flourish

from schooltool.rising import RisingMessage as _


class IStreams(Interface):

    stream1 = Choice(
        title=_('Advisory group'),
        source='schooltool.stream.streams')

    stream2 = Choice(
        title=_('Morning class'),
        source='schooltool.stream.streams')

    stream3 = Choice(
        title=_('Afternoon class'),
        source='schooltool.stream.streams')


class StudentAddView(ArkStudentAddView):

    def buildFieldsetGroups(self):
        super(StudentAddView, self).buildFieldsetGroups()
        self.fieldset_groups['streams'] = (
                _('Streams'), list(self.getStreamsFields()))
        self.fieldset_order = (
            'full_name', 'details', 'demographics', 'streams', 'level',
            'relationships', 'user')

    def getStreamsFields(self):
        fields = field.Fields(IStreams)
        return fields

    def update(self):
        self.buildFieldsetGroups()
        self.fields = self.getBaseFields()
        self.fields += self.getDemoFields()
        self.fields += self.getStreamsFields()
        self.updateWidgets()
        self.updateActions()
        self.actions.execute()
        self.widgets['birth_date'].addClass('birth-date-field')

    def create(self, data):
        self._stream1 = data.pop('stream1')
        self._stream2 = data.pop('stream2')
        self._stream3 = data.pop('stream3')
        return super(StudentAddView, self).create(data)

    def add(self, person):
        person = super(StudentAddView, self).add(person)
        today = getUtility(IDateManager).today
        date = self._group_date if self._group_date is not None else today
        streams = [self._stream1, self._stream2, self._stream3]
        for stream in streams:
            if person not in stream.members.on(date):
                stream.members.on(date).add(person)
        return person


class AddStudentLinkViewlet(flourish.page.LinkViewlet):

    @property
    def url(self):
        return "%s/%s" % (absoluteURL(self.context, self.request),
                          'addRisingStudent.html')
