#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2015 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Configuration handler
"""

from zope.component import provideUtility

from schooltool.rising.access import JourneyAccessUtility


def handle_configuration(options, context):
    journey = getattr(options.config, 'journey', None)
    if journey:
        utility = JourneyAccessUtility()
        utility.api = journey.api
        utility.username = journey.username
        utility.password = journey.password
        provideUtility(utility)
        context.provideFeature('journey')


def get_configuration():
    return ("""
<import package="schooltool.rising" />

<section type="journey" name="*" required="no" attribute="journey">
  <description>
    Configuration for Journey API access
  </description>
  <example>
    &lt;journey&gt;
      api https://somename.journeyapps.com/api/v4/
      username 1234567890
      password 1234567890
    &lt;/journey&gt;
  </example>
</section>

""", handle_configuration)
