#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2015 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Rising demographics initialization
"""

from schooltool.app.app import InitBase
from schooltool.basicperson.demographics import BoolFieldDescription
from schooltool.basicperson.demographics import EnumFieldDescription
from schooltool.basicperson.demographics import IntFieldDescription
from schooltool.basicperson.demographics import TextFieldDescription
from schooltool.contact.interfaces import IContactDemographicsFields

from schooltool.rising import RisingMessage as _


def setUpContactDemographics(app):
    dfs = IContactDemographicsFields(app)

    if 'gender' not in dfs:
        dfs['gender'] = EnumFieldDescription(
            'gender', _('Gender'))
        dfs['gender'].items = [
            _('Male'),
            _('Female'),
        ]

    if 'age' not in dfs:
        dfs['age'] = EnumFieldDescription(
            'age', _('Age'))
        dfs['age'].items = [
            _('Less than 20'),
            _('20-29'),
            _('30-39'),
            _('40-49'),
            _('50-59'),
            _('60 or more'),
        ]

    if 'main_contact' not in dfs:
        dfs['main_contact'] = BoolFieldDescription(
            'main_contact', _('Main contact point'))

    if 'fees_contact' not in dfs:
        dfs['fees_contact'] = BoolFieldDescription(
            'fees_contact',
            _('Responsible for paying school fees for student?'))

    if 'contact_mobile_phone' not in dfs:
        dfs['contact_mobile_phone'] = TextFieldDescription(
            'contact_mobile_phone', _('Mobile telephone'))

    if 'contact_home_phone' not in dfs:
        dfs['contact_home_phone'] = TextFieldDescription(
            'contact_home_phone', _('Home telephone'))

    if 'contact_email' not in dfs:
        dfs['contact_email'] = TextFieldDescription(
            'contact_email', _('Email'))

    if 'address_house' not in dfs:
        dfs['address_house'] = TextFieldDescription(
            'address_house', _('Address: House number and street'))

    if 'address_area' not in dfs:
        dfs['address_area'] = TextFieldDescription(
            'address_area', _('Address: Area/Community'))

    if 'address_town' not in dfs:
        dfs['address_town'] = TextFieldDescription(
            'address_town', _('Address: Town'))

    if 'address_district' not in dfs:
        dfs['address_district'] = TextFieldDescription(
            'address_district', _('Address: District'))

    if 'attended_school' not in dfs:
        dfs['attended_school'] = BoolFieldDescription(
            'attended_school', _('Ever attended school'))

    if 'highest_school_level' not in dfs:
        dfs['highest_school_level'] = EnumFieldDescription(
            'highest_school_level', _('If yes, highest level attained'))
        dfs['highest_school_level'].items = [
            _('Primary'),
            _('Secondary'),
            _('Higher'),
        ]

    if 'primary_qualification' not in dfs:
        dfs['primary_qualification'] = TextFieldDescription(
            'primary_qualification', _('Primary'),
            description=_('specify form/qualification completed'))

    if 'secondary_qualification' not in dfs:
        dfs['secondary_qualification'] = TextFieldDescription(
            'secondary_qualification', _('Secondary'),
            description=_('specify form/qualification completed'))

    if 'higher_qualification' not in dfs:
        dfs['higher_qualification'] = TextFieldDescription(
            'higher_qualification', _('Higher'),
            description=_('specify form/qualification completed'))

    if 'currently_employed' not in dfs:
        dfs['currently_employed'] = BoolFieldDescription(
            'currently_employed', _('Currently employed'),
            description=_('if no, skip to unemployed follow up'))

    if 'how_often_paid' not in dfs:
        dfs['how_often_paid'] = EnumFieldDescription(
            'how_often_paid', _('Current occupation - How often paid'))
        dfs['how_often_paid'].items = [
            _('Daily'),
            _('Weekly'),
            _('Monthly'),
            _('Varies/unknown'),
        ]

    if 'how_much_paid' not in dfs:
        dfs['how_much_paid'] = EnumFieldDescription(
            'how_much_paid', _('Current occupation - How much paid'))
        dfs['how_much_paid'].items = [
            _('Different amounts'),
            _('Same amounts'),
        ]

    if 'unemployment_desired' not in dfs:
        dfs['unemployment_desired'] = BoolFieldDescription(
            'unemployment_desired',
            _('Unemployed: is unemployment desired or undesired'))

    if 'unemployment_reason' not in dfs:
        dfs['unemployment_reason'] = TextFieldDescription(
            'unemployment_reason',
            _('Unemployed: specify reason for not working'))

    if 'single_parent' not in dfs:
        dfs['single_parent'] = BoolFieldDescription(
            'single_parent', _('Single parent/caregiver?'))

    if 'household_total' not in dfs:
        dfs['household_total'] = IntFieldDescription(
            'household_total',
            _('Total number living in the household'))

    if 'household_adults' not in dfs:
        dfs['household_adults'] = IntFieldDescription(
            'household_adults', _('Number of adults'))

    if 'household_children' not in dfs:
        dfs['household_children'] = IntFieldDescription(
            'household_children', _('Number of children'))

    if 'household_dependents' not in dfs:
        dfs['household_dependents'] = IntFieldDescription(
            'household_dependents', _('Total number of financial dependents'))

    if 'household_fees' not in dfs:
        dfs['household_fees'] = IntFieldDescription(
            'household_fees',
            _("Total number of other children's school fees being paid"))

    if 'household_rising' not in dfs:
        dfs['household_rising'] = IntFieldDescription(
            'household_rising',
            _('Number of other children attending Rising schools'))

    if 'current_home' not in dfs:
        dfs['current_home'] = TextFieldDescription(
            'current_home', _('How long lived in current home?'))

    if 'own_rent' not in dfs:
        dfs['own_rent'] = EnumFieldDescription(
            'own_rent', _('Own or rent the home?'))
        dfs['own_rent'].items = [
            _('Own'),
            _('Rent'),
        ]

    if 'hardship_index' not in dfs:
        dfs['hardship_index'] = TextFieldDescription(
            'hardship_index', _('Hardship Index Score'))

    if 'heard_rising' not in dfs:
        dfs['heard_rising'] = TextFieldDescription(
            'heard_rising', _('How heard about Rising?'))

    if 'concerns' not in dfs:
        dfs['concerns'] = TextFieldDescription(
            'concerns',
            _("Any concerns about child's education progress so far"))


class RisingContactDemographicsInit(InitBase):

    after = ('schooltool.contact.demographics',)

    def __call__(self):
        setUpContactDemographics(self.app)
