#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2015 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Rising demographics initialization
"""

from schooltool.app.app import InitBase
from schooltool.basicperson.demographics import BoolFieldDescription
from schooltool.basicperson.demographics import DateFieldDescription
from schooltool.basicperson.demographics import DescriptionFieldDescription
from schooltool.basicperson.demographics import EnumFieldDescription
from schooltool.basicperson.demographics import IntFieldDescription
from schooltool.basicperson.demographics import TextFieldDescription
from schooltool.basicperson.interfaces import IDemographicsFields

from schooltool.rising import RisingMessage as _


def setUpDemographics(app):
    dfs = IDemographicsFields(app)

    if 'goes_by' not in dfs:
        dfs['goes_by'] = TextFieldDescription(
            'goes_by', _('Goes by'),
            limit_keys=['students'])

    if 'scholarship_student' not in dfs:
        dfs['scholarship_student'] = EnumFieldDescription(
            'scholarship_student', _('Scholarship student?'),
            limit_keys=['students'])
        dfs['scholarship_student'].items = [
            _('No'),
            _('Rising Scholar'),
            _('Community Scholar'),
            _('Hardship Fund Recipient'),
        ]

    if 'language' not in dfs:
        dfs['language'] = EnumFieldDescription(
            'language', _('Main language spoken at home'),
            limit_keys=['students'])
        dfs['language'].items = [
            _('English'),
            _('Krio'),
            _('Temne'),
            _('Mende'),
            _('Limba'),
            _('Kono'),
            _('Kissi'),
            _('Kuranko'),
            _('Fula'),
            _('Other'),
        ]

    if 'school_code' not in dfs:
        dfs['school_code'] = TextFieldDescription(
            'school_code', _('School Code'),
            limit_keys=['teachers'])

    if 'shift_type' not in dfs:
        dfs['shift_type'] = EnumFieldDescription(
            'shift_type', _('Shift type'),
            limit_keys=['teachers'], required=True)
        dfs['shift_type'].items = [_('Full time'),
                                   _('Part time')]

    if 'employment_date' not in dfs:
        dfs['employment_date'] = DateFieldDescription(
            'employment_date', _('Date of first employment at Rising'),
            limit_keys=['teachers'], required=True)

    if 'language_other' not in dfs:
        dfs['language_other'] = TextFieldDescription(
            'language_other', _('If other, please specify'),
            limit_keys=['students'])

    if 'subject_1' not in dfs:
        dfs['subject_1'] = TextFieldDescription(
            'subject_1', _('Subject specialism 1'),
            limit_keys=['teachers'])

    if 'subject_2' not in dfs:
        dfs['subject_2'] = TextFieldDescription(
            'subject_2', _('Subject specialism 2'),
            limit_keys=['teachers'])

    if 'qualifications' not in dfs:
        dfs['qualifications'] = EnumFieldDescription(
            'qualifications', _('Qualifications'),
            limit_keys=['teachers'])
        dfs['qualifications'].items = [_('Graduate'),
                                       _('Diploma'),
                                       _('S6 graduate'),
                                       _('Unqualified')]

    if 'other_responsibilities' not in dfs:
        dfs['other_responsibilities'] = EnumFieldDescription(
            'other_responsibilities', _('Other Responsibilities'),
            limit_keys=['teachers'])
        dfs['other_responsibilities'].items = [
            _('Academy Manager'),
            _('Master Teacher'),
            _('Senior Teacher'),
            _('Class Teacher'),
            _('SchoolTool core team'),
        ]

    if 'cpd_attended' not in dfs:
        dfs['cpd_attended'] = DescriptionFieldDescription(
            'cpd_attended', _('CPD attended'),
            limit_keys=['teachers'])

    if 'email' not in dfs:
        dfs['email'] = TextFieldDescription(
            'email', _('Email'),
            limit_keys=['teachers'])

    if 'religion' not in dfs:
        dfs['religion'] = EnumFieldDescription(
            'religion', _('Religion'),
            limit_keys=['students'])
        dfs['religion'].items = [
            _('Muslim'),
            _('Christian'),
            _('Other'),
        ]

    if 'phone' not in dfs:
        dfs['phone'] = TextFieldDescription(
            'phone', _('Phone'),
            limit_keys=['students', 'teachers'])

    if 'disability' not in dfs:
        dfs['disability'] = TextFieldDescription(
            'disability', _('Disability?'),
            limit_keys=['students'])

    if 'dietary_medical' not in dfs:
        dfs['dietary_medical'] = TextFieldDescription(
            'dietary_medical', _('Dietary requirements / medical conditions'),
            limit_keys=['students'])

    if 'distance' not in dfs:
        dfs['distance'] = IntFieldDescription(
            'distance', _('Distance travelled to school (minutes)'),
            limit_keys=['students'])

    if 'transport' not in dfs:
        dfs['transport'] = EnumFieldDescription(
            'transport', _('Typical mode of transport'),
            limit_keys=['students'])
        dfs['transport'].items = [
            _('Walk only'),
            _('Okada'),
            _('Poda-poda/taxi'),
            _('Private car (driven by family member)'),
        ]
        
    if 'primary_school' not in dfs:
        dfs['primary_school'] = TextFieldDescription(
            'primary_school', _('Primary school name'),
            limit_keys=['students'])

    if 'primary_school_type' not in dfs:
        dfs['primary_school_type'] = EnumFieldDescription(
            'primary_school_type', _('Primary school type'),
            limit_keys=['students'])
        dfs['primary_school_type'].items = [
            _('Government'),
            _('NGO/Charity'),
            _('Church'),
            _('Mosque'),
            _('Private'),
            _('Community'),
        ]

    if 'out_of_school' not in dfs:
        dfs['out_of_school'] = BoolFieldDescription(
            'out_of_school',
            _('Out of school previously?'),
            limit_keys=['students'])

    if 'out_of_school_reason' not in dfs:
        dfs['out_of_school_reason'] = EnumFieldDescription(
            'out_of_school_reason', _('If yes, reason for being out of school'),
            limit_keys=['students'])
        dfs['out_of_school_reason'].items = [
            _('Left to have a baby (pregnancy)'),
            _('Work'),
            _('Fees'),
            _('Family death or illness'),
        ]

    if 'primary_exam_aggregate' not in dfs:
        dfs['primary_exam_aggregate'] = TextFieldDescription(
            'primary_exam_aggregate', _('Primary exam - Aggregate'),
            limit_keys=['students'])

    if 'primary_exam_mathematics' not in dfs:
        dfs['primary_exam_mathematics'] = TextFieldDescription(
            'primary_exam_mathematics', _('Primary exam - Mathematics'),
            limit_keys=['students'])

    if 'primary_exam_english' not in dfs:
        dfs['primary_exam_english'] = TextFieldDescription(
            'primary_exam_english', _('Primary exam - English'),
            limit_keys=['students'])

    if 'primary_exam_quantitative_reasoning' not in dfs:
        dfs['primary_exam_quantitative_reasoning'] = TextFieldDescription(
            'primary_exam_quantitative_reasoning',
            _('Primary exam - Quantitative reasoning'),
            limit_keys=['students'])

    if 'primary_exam_verbal_reasoning' not in dfs:
        dfs['primary_exam_verbal_reasoning'] = TextFieldDescription(
            'primary_exam_verbal_reasoning',
            _('Primary exam - Verbal reasoning'),
            limit_keys=['students'])

    if 'primary_exam_general_studies' not in dfs:
        dfs['primary_exam_general_studies'] = TextFieldDescription(
            'primary_exam_general_studies',
            _('Primary exam - General studies'),
            limit_keys=['students'])

    if 'address_house' not in dfs:
        dfs['address_house'] = TextFieldDescription(
            'address_house', _('Address: House number and street'),
            limit_keys=['students', 'teachers'])

    if 'address_area' not in dfs:
        dfs['address_area'] = TextFieldDescription(
            'address_area', _('Address: Area/Community'),
            limit_keys=['students', 'teachers'])

    if 'address_town' not in dfs:
        dfs['address_town'] = TextFieldDescription(
            'address_town', _('Address: Town'),
            limit_keys=['students', 'teachers'])

    if 'address_district' not in dfs:
        dfs['address_district'] = TextFieldDescription(
            'address_district', _('Address: District'),
            limit_keys=['students', 'teachers'])


class RisingDemographicsInit(InitBase):

    after = ('schooltool.basicperson.demographics',)

    def __call__(self):
        self.deleteDefaultFields()
        setUpDemographics(self.app)

    def deleteDefaultFields(self):
        dfs = IDemographicsFields(self.app)
        names = ['ID', 'ethnicity', 'language', 'placeofbirth', 'citizenship']
        for name in names:
            if name in dfs:
                del dfs[name]
