#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2015 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Evolve database to generation 1.

Adds new contact demographics fields
"""

from zope.app.generations.utility import findObjectsProviding
from zope.app.publication.zopepublication import ZopePublication
from zope.component.hooks import getSite, setSite

from schooltool.app.interfaces import ISchoolToolApplication
from schooltool.requirement.interfaces import IEvaluations
from schooltool.lyceum.journal.journal import StudentAMRequirement
from schooltool.lyceum.journal.journal import StudentPMRequirement
from schooltool.lyceum.journal.journal import TeacherAMRequirement
from schooltool.lyceum.journal.journal import TeacherPMRequirement
from schooltool.lyceum.journal.browser.journal import makeSchoolAttendanceMeeting
from schooltool.lyceum.journal.interfaces import IJournalScoreSystemPreferences
from schooltool.lyceum.journal.interfaces import IEvaluateRequirement
from schooltool.requirement.scoresystem import UNSCORED


def evolve(context):
    root = context.connection.root().get(ZopePublication.root_name, None)
    old_site = getSite()
    apps = list(findObjectsProviding(root, ISchoolToolApplication))
    for app in apps:
        setSite(app)
        teacher_ss = IJournalScoreSystemPreferences(app).teacher_scoresystem
        student_ss = IJournalScoreSystemPreferences(app).student_scoresystem
        requirement_factories = [
            (StudentAMRequirement, student_ss),
            (StudentPMRequirement, student_ss),
            (TeacherAMRequirement, teacher_ss),
            (TeacherPMRequirement, teacher_ss),
        ]
        persons = app['persons']
        evaluator = persons['manager']
        for person in persons.values():
            evaluations = IEvaluations(person)
            requirements = evaluations.keys()
            for requirement in requirements:
                for factory, ss in requirement_factories:
                    if isinstance(requirement, factory):
                        score = IEvaluateRequirement(requirement).getEvaluation(
                            person, requirement, default=UNSCORED)
                        if (score is UNSCORED or score.value is UNSCORED):
                            continue
                        if requirement.score_system is not ss:
                            meeting = makeSchoolAttendanceMeeting(
                                requirement[1])
                            new_requirement = factory(meeting, ss)
                            new_score = [
                                value for value, title in ss.scores
                                if value.lower() == score.value.lower()]
                            if new_score:
                                IEvaluateRequirement(new_requirement).evaluate(
                                    person, new_requirement, new_score[0],
                                    evaluator=evaluator)
    setSite(old_site)
