#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2016 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Evolve database to generation 3.

Adds score system preferences
"""

from zope.app.generations.utility import getRootFolder
from zope.component.hooks import getSite, setSite
from zope.container.interfaces import INameChooser

from schooltool.requirement.interfaces import IScoreSystemContainer
from schooltool.lyceum.journal.interfaces import IJournalScoreSystemPreferences
from schooltool.lyceum.journal.journal import GlobalAbsenceScoreSystem
from schooltool.lyceum.journal.journal import PersistentAttendanceScoreSystem

from schooltool.rising import RisingMessage as _


StudentAbsenceScoreSystem = GlobalAbsenceScoreSystem(
    'StudentAbsenceScoreSystem',
    _('Student Absences'),
    scores=(('P', _('Present')),
            ('Pl', _('Present but late and / or left early')),
            ('As', _('Absent - sick')),
            ('Ao', _('Absent - other reason')),
            ('Au', _('Absent - unknown')),
            ('Sb', _('Suspended - behaviour')),
            ('Sf', _('Suspended - fees')),
           ),
    tag_absent=('As', 'Ao', 'Au'),
    tag_tardy=(),
    tag_excused=(),
    tag_skip=('Sb', 'Sf'),
    )


TeacherAbsenceScoreSystem = GlobalAbsenceScoreSystem(
    'TeacherAbsenceScoreSystem',
    _('Teacher Absences'),
    scores=(('P', _('Present')),
            ('Pl-a', _('Present but late and / or left early - authorised')),
            ('Pl-u', _('Present but late and / or left early - unauthorised')),
            ('As', _('Absent - sick')),
            ('Aa', _('Absent - authorised')),
            ('An', _('Absent - non-authorised')),
            ('S', _('Suspended')),
            ('NT', _('Not timetabled')),
           ),
    tag_absent=('As', 'Aa', 'An'),
    tag_tardy=(),
    tag_excused=(),
    tag_skip=('S', 'NT'),
    )


def evolve(context):
    old_site = getSite()
    app = getRootFolder(context)
    setSite(app)
    container = IScoreSystemContainer(app)
    chooser = INameChooser(container)
    prefs = IJournalScoreSystemPreferences(app)
    if prefs.teacher_scoresystem is None:
        scores = TeacherAbsenceScoreSystem.scores
        tag_absent = TeacherAbsenceScoreSystem.tag_absent
        tag_tardy = TeacherAbsenceScoreSystem.tag_tardy
        tag_excused = TeacherAbsenceScoreSystem.tag_excused
        tag_skip = TeacherAbsenceScoreSystem.tag_skip
        ss = PersistentAttendanceScoreSystem(TeacherAbsenceScoreSystem.title,
                                             scores=scores,
                                             tag_absent=tag_absent,
                                             tag_tardy=tag_tardy,
                                             tag_excused=tag_excused,
                                             tag_skip=tag_skip)
        name = chooser.chooseName(ss.title, ss)
        container[name] = ss
        prefs.teacher_scoresystem = ss
    if prefs.student_scoresystem is None:
        scores = StudentAbsenceScoreSystem.scores
        tag_absent = StudentAbsenceScoreSystem.tag_absent
        tag_tardy = StudentAbsenceScoreSystem.tag_tardy
        tag_excused = StudentAbsenceScoreSystem.tag_excused
        tag_skip = StudentAbsenceScoreSystem.tag_skip
        ss = PersistentAttendanceScoreSystem(StudentAbsenceScoreSystem.title,
                                             scores=scores,
                                             tag_absent=tag_absent,
                                             tag_tardy=tag_tardy,
                                             tag_excused=tag_excused,
                                             tag_skip=tag_skip)
        name = chooser.chooseName(ss.title, ss)
        container[name] = ss
        prefs.student_scoresystem = ss
    setSite(old_site)
