#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2015 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Rising scoresystem
"""

from decimal import Decimal

from zope.cachedescriptors.property import Lazy
from zope.component import adapter
from zope.component import adapts
from zope.interface import implementer
from zope.interface import implements

from schooltool.app.app import StartUpBase
from schooltool.app.interfaces import ISchoolToolApplication
from schooltool.ark.interfaces import ITargetSheetScoreSystem
from schooltool.ark.interfaces import IScoreSystemColors
from schooltool.requirement.interfaces import IScoreSystemContainer
from schooltool.requirement.interfaces import IDiscreteValuesScoreSystem
from schooltool.requirement.scoresystem import CustomScoreSystem
from schooltool.requirement.scoresystem import GlobalDiscreteValuesScoreSystem

from schooltool.rising import RisingMessage as _


RISING_TARGET_SCORESYSTEM_KEY = 'schooltool.rising.target_scoresystem'


RisingTargetScoreSystem = GlobalDiscreteValuesScoreSystem(
    'Rising Target Score System',
    _('Rising Target Score System'), None,
    [
        ('6', u'Grade 6', Decimal(6), Decimal(85)),
        ('5', u'Grade 5', Decimal(5), Decimal(70)),
        ('4', u'Grade 4', Decimal(4), Decimal(55)),
        ('3', u'Grade 3', Decimal(3), Decimal(40)),
        ('2', u'Grade 2', Decimal(2), Decimal(25)),
        ('1', u'Grade 1', Decimal(1), Decimal(0)),
    ],
    '6', '2', False)


class RisingScoreSystemAppStartup(StartUpBase):

    after = ('schooltool.requirement.scoresystem',)

    def __call__(self):
        container = IScoreSystemContainer(self.app)
        if RISING_TARGET_SCORESYSTEM_KEY not in container:
            ss = RisingTargetScoreSystem
            rising_scoresystem = CustomScoreSystem(ss.title, ss.description,
                                                   ss.scores, ss._bestScore,
                                                   ss._minPassingScore,
                                                   ss._isMaxPassingScore)
            container[RISING_TARGET_SCORESYSTEM_KEY] = rising_scoresystem


@adapter(ISchoolToolApplication)
@implementer(ITargetSheetScoreSystem)
def getTargetScoreSystem(app):
    container = IScoreSystemContainer(app)
    return container.get(RISING_TARGET_SCORESYSTEM_KEY)


class ScoreSystemColors(object):

    implements(IScoreSystemColors)
    adapts(IDiscreteValuesScoreSystem)

    def __init__(self, context):
        self.context = context

    @Lazy
    def colors(self):
        return {
            '6': {'html': 'green'     , 'xlwt': 'bright_green'},
            '5': {'html': 'lightgreen', 'xlwt': 'light_green'},
            '4': {'html': 'yellow'    , 'xlwt': 'yellow'},
            '3': {'html': 'orange'    , 'xlwt': 'orange'},
            '2': {'html': 'orange'    , 'xlwt': 'orange'},
            '1': {'html': 'red'       , 'xlwt': 'red'},
        }


class RisingTeacherStateToAttendance(object):

    states = {
        'a': 'p',
        'l': 'pl-a',
        'c': 'pl-u',
        's': 'as',
        'z': 'aa',
        'i': 'an',
        'u': 's',
        't': 'nt',
        }
